<?php

/**
 * @package         Convert Forms
 * @version         4.4.11-RC3 Free
 * 
 * @author          Tassos Marinos <info@tassos.gr>
 * @link            https://www.tassos.gr
 * @copyright       Copyright © 2025 Tassos All Rights Reserved
 * @license         GNU GPLv3 <http://www.gnu.org/licenses/gpl.html> or later
 */

namespace ConvertForms\Validation\Rules;

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\User\UserHelper;

class Honeypot extends \ConvertForms\Validation\Rule
{
    private $sessionProp = 'convertforms.honeypot';

    public function validate()
    {
        if (!$this->isEnabled())
        {
            return;
        }
     
        $honeypotId = $this->getSessionId();

        // If empty, might be a direct attempt to submit the form.
        if (is_null($honeypotId))
        {
            $this->setError('Session ID not found');
            return false;
        }

        // The honeypot field should be present in the submission data. Otherwise might be a direct attempt.
        if (!isset($this->submission[$honeypotId]))
        {
            $this->setError('Honeypot field not found');
            return false;
        }

        // The honeypot field should be empty. Otherwise it's likely a bot.
        if (!empty($this->submission[$honeypotId]))
        {
            return false;
        }
    }

    public function onFormBeforeRender()
    {
        if (!$this->isEnabled())
        {
            return;
        }

        $boxAtts = isset($this->form['containerAtts']) ? $this->form['containerAtts'] : [];

        // Let's add some data attributes for the client-side script.
        $this->form['containerAtts'] = array_merge($boxAtts, [
            // Fire the session XHR request which fires the onInit() method
            'data-cf-ss' => '',

            // Let the script know this form needs a honeypot field.
            'data-cf-hp' => ''
        ]);
    }

    public function onInit()
    {
        if (!$this->getSessionId())
        {
            $id = $this->createRandomString();
            $this->app->getSession()->set($this->sessionProp, $id);
        }
    }

    private function getSessionId()
    {
        return $this->app->getSession()->get($this->sessionProp, null);
    }

    // Creates a random string with an alphabetic character in the beginning (required for a valid CSS name)
    private function createRandomString()
    {
        $charactersArray = array_merge(range('a', 'z'), range('A', 'Z'));
        $randomString = $charactersArray[random_int(0, count($charactersArray) - 1)];
        $randomString .= UserHelper::genRandomPassword(random_int(7, 19));

        return $randomString;
    }

    /**
     * Returns the minimum required time in seconds. Defaults to 2 seconds.
     *
     * @return int
     */
    private function isEnabled()
    {
        return (int) $this->getFormRegistry()->get('params.honeypot') == 2;
    }
}